@@ -1,6 +1,6 @@
 package OpenInteract::Handler::Page;
 
-# $Id: Page.pm,v 1.30 2002/01/18 20:07:49 lachoy Exp $
+# $Id: Page.pm,v 1.37 2002/02/15 22:03:59 lachoy Exp $
 
 use strict;
 use Class::Date ();
@@ -9,7 +9,7 @@ use OpenInteract::Handler::GenericDispatcher qw( DEFAULT_SECURITY_KEY );
 use SPOPS::Secure qw( :level );
 
 @OpenInteract::Handler::Page::ISA     = qw( OpenInteract::CommonHandler  SPOPS::Secure );
-$OpenInteract::Handler::Page::VERSION = sprintf("%d.%02d", q$Revision: 1.30 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::Page::VERSION = sprintf("%d.%02d", q$Revision: 1.37 $ =~ /(\d+)\.(\d+)/);
 $OpenInteract::Handler::Page::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::Page::default_method    = 'show';
 @OpenInteract::Handler::Page::forbidden_methods = ();
@@ -53,7 +53,14 @@ sub MY_OBJECT_CREATE_SECURITY  { return SEC_LEVEL_WRITE }
 
 sub MY_EDIT_RETURN_URL         { return '/' }
 sub MY_EDIT_FAIL_TASK          { return 'show' }
-sub MY_EDIT_DISPLAY_TASK       { return 'show' }
+
+# Use this if you want to display the page just edited
+#sub MY_EDIT_DISPLAY_TASK       { return 'show' }
+
+# Use this if you want to display the status of the page just edited
+# -- good for uploads vs. in-place edits
+sub MY_EDIT_DISPLAY_TASK       { return '_edit_status' }
+
 sub MY_EDIT_FIELDS             { return qw( location title author keywords
                                             boxes main_template notes content
                                             storage content_location ) }
@@ -78,6 +85,20 @@ sub actions {
                                   { name => 'base_page::admin_actions' } );
 }
 
+
+my %HELP_MAP = ( upload => 'page_upload_help' );
+
+sub help {
+    my ( $class, $p ) = @_;
+    my $R = OpenInteract::Request->instance;
+    my $type = $class->_read_field( $R->apache, 'type' ) || 'upload';
+    my $tmpl_name = $HELP_MAP{ $type };
+    $R->{page}{title} = "Page Administrative Actions";
+    $R->{page}{_template_name_} = 'base_page::page_base_help';
+    return $R->template->handler( {}, {},
+                                  { name => "base_page::$tmpl_name" } );
+}
+
 # Overrides entry in OpenInteract::Handler::GenericDispatcher
 
 sub _get_task {
@@ -149,6 +170,18 @@ sub directory_list {
 }
 
 
+# Override to first check for 'old_location' and THEN the normal ID
+
+sub fetch_object {
+    my ( $class, $id, @id_field ) = @_;
+    my $R = OpenInteract::Request->instance;
+    my $existing_location = $class->_read_field( $R->apache, 'old_location' );
+    return ( $existing_location )
+             ? $class->SUPER::fetch_object( $existing_location, @id_field )
+             : $class->SUPER::fetch_object( $id, @id_field );
+}
+
+
 # Yes, this is out of the normal order. It's just that show() is so
 # big and includes so much stuff...
 
@@ -180,9 +213,20 @@ sub _edit_customize {
             return ( ERROR, $error_message );
         }
         $dir =~ s|/$||;
-        my $filename = $R->page->clean_location( $page->{tmp_upload}{filename} );
-        $page->{location} = join( '', $dir, $filename );
-        $R->DEBUG && $R->scrib( 1, "Set location from uploaded file to [$page->{location}]" );
+        my $filename = $R->page->clean_filename( $page->{tmp_upload}{filename} );
+        $page->{location} = join( '/', $dir, $filename );
+        $R->DEBUG && $R->scrib( 1, "Set location to [$page->{location}] from",
+                                   "filename reported by client [$filename]" );
+
+        # Check to see if this location is already in the
+        # database. (For non-simple pages this is done in the normal
+        # editing sequence.) If so, set the saved-status of this to
+        # true so the update takes place.
+
+        unless ( $page->is_saved ) {
+            my $existing_page = eval { $R->page->fetch( $page->{location} ) };
+            $page->has_save if ( $existing_page );
+        }
     }
 
     # If we're not using the simple form, check to see if the user
@@ -191,6 +235,9 @@ sub _edit_customize {
 
     elsif ( $page->{location} ne $old_data->{location} ) {
         $opts{use_id} = $old_data->{location};
+        $R->DEBUG && $R->scrib( 1, "This should be an UPDATE where we change ",
+                                   "the primary key from [$old_data->{location}] ",
+                                   "to [$page->{location}]" );
     }
 
     # Ensure that the location is clean
@@ -224,6 +271,28 @@ sub _edit_customize {
     return ( OK, \%opts );
 }
 
+# If this is a successful update and the item's location has been
+# changed, we need to tell the content implementation class to rename
+# the content and do whatever other actions it requires.
+
+sub _edit_post_action_customize {
+    my ( $class, $page, $old_data ) = @_;
+    if ( $page->{location} ne $old_data->{location} ) {
+        $page->rename_content( $old_data->{location} );
+    }
+    return 1;
+}
+
+
+sub _edit_status {
+    my ( $class, $p ) = @_;
+    my $R = OpenInteract::Request->instance;
+    $p->{directory} = $p->{page}{directory};
+    $p->{location}  = $p->{page}{location};
+    $R->{page}{title} = 'Status of page edit';
+    return $R->template->handler( {}, $p, { name => 'base_page::page_edit_status' } );
+}
+
 
 # Why do we set the content-type when returning errors? See note on
 # error content-type forcing in POD...
